import requests
import json


def make_header(token):
    return {
        "Authorization": f'Zoho-oauthtoken {token.access}'
    }

def get_territories(token):
    url = "https://www.zohoapis.com/crm/v2.1/settings/territories"
    headers = make_header(token)
    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        print("Auth")
        return get_territories(token)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("territories")


def assign_territories(token, module, record_id, territory_id_list):
    url = f"https://www.zohoapis.com/crm/v2.1/{module}/actions/assign_territories"
    headers = make_header(token)
    request_body = {}

    data_object = {'id': record_id}
    territory_list = []
    for id in territory_id_list:
        territory_list.append({"id":id})

    data_object['Territories'] = territory_list
    record_list = [data_object]

    request_body['data'] = record_list

    data = json.dumps(request_body).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return assign_territories(token, module, record_id, territory_id_list)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content



def remove_territories(token, module, record_id, territory_id_list):
    url = f"https://www.zohoapis.com/crm/v2.1/{module}/actions/remove_territories"
    headers = make_header(token)
    request_body = {}

    data_object = {'id': record_id}
    territory_list = []
    for id in territory_id_list:
        territory_list.append({"id":id})

    data_object['Territories'] = territory_list
    record_list = [data_object]

    request_body['data'] = record_list

    data = json.dumps(request_body).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return remove_territories(token, module, record_id, territory_id_list)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content


def get_territories_assigned(token, module, record_id):
    url = f'https://www.zohoapis.com/crm/v2.1/{module}/{record_id}'
    headers = make_header(token)
    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return get_territories_assigned(token, module, record_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        data = content.get("data")
        if data is not None:
            return token, data.get("Territories")
    



import requests
import json


def format_header(token):
    return {
        "Authorization": "Zoho-oauthtoken {token.access}"
    }


def create_contact_person(token, org_id, contact_id, data_object):
    url = f'https://books.zoho.com/api/v3/contacts/contactpersons?organization_id={org_id}'
    headers = format_header(token)

    data_object['contact_id'] = contact_id

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 400:
        token.generate()
        return create_contact_person(token, org_id, contact_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        try:
            contact_person = content['contact_person']

        except Exception as e:
            contact_person = {'status': response.status_code, "error": str(e)}

        return token, response.status_code, contact_person



def update_contact_person(token, org_id, contact_person_id, data_object):
    url = f'https://books.zoho.com/api/v3/contacts/contactpersons/{contact_person_id}?organization_id={org_id}'
    headers = format_header(token)

    

    data = json.dumps(data_object).encode('utf-8')

    response = requests.put(url=url, headers=headers, data=data)

    if response.status_code == 400:
        token.generate()
        return create_contact_person(token, org_id, contact_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        try:
            contact_person = content['contact_person']

        except Exception as e:
            contact_person = {'status': response.status_code, "error": str(e)}

        return token, response.status_code, contact_person



def delete_contact_person(token, org_id, contact_person_id):
    url = f'https://books.zoho.com/api/v3/contacts/contactpersons/{contact_person_id}?organization_id={org_id}'
    headers = format_header(token)

    response = requests.delete(url=url, headers=headers)

    if response.status_code == 400:
        token.generate()
        return delete_contact_person(token, org_id, contact_person_id)

    else:
        content = json.loads(resonse.content.decode('utf-8'))
        return token, content['message']


def get_contact_persons(token, org_id, contact_id, **kwargs):
    url = f'https//books.zoho.com/api/v3/contacts/{contact_id}/contactpersons?organization_id={org_id}'
    headers = format_header(token)

    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 400:
        token.generate()
        return get_contact_persons(token, org_id, contact_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
    	try:
        	contact_persons = content['contact_persons']
        	page_context = content['page_context']
    	except Exception as e:
        	contact_persons = [{"status": response.status_code, "error": str(e)}]
       		page_context = None
    
   		return token, page_context, contact_persons
    
    
def get_contact_person(token, org_id, contact_id, contact_person_id):
	url = f'https://books.zoho.com/api/v3/contacts/{contact_id}/contactpersons/{contact_person_id}?organization_id={org_id}'
	headers = format_header(token)
	
	response = requests.get(url=url, headers=headers)
	
	if response.status_code == 400:
		token.generate()
		return get_contact_person(token, org_id, contact_id, contact_person_id)
		
	else:
		content = json.loads(response.content.decode('utf-8'))
		try:
			contact_person = content['contact_person']
		except Exception as e:
			contact_person = {'status': response.status_code, 'error': str(e)}
		return token, contact_person
		


		




import requests 
import json
from datetime import datetime


def make_header(token):
    return {
        "Authorization": f"Zoho-oauthtoken {token.access}"
    }


def get_record(token, module, record_id):
    url = f"https://www.zohoapis.com/crm/v2.1/{module}/{record_id}"
    headers = make_header(token)
    response = requests.get(url=url, headers=headers)
    if response.status_code == 401:
        print("Authentication issue")
        token.generate()
        return get_record(token, module, record_id)

    else:
        json_content = json.loads(response.content.decode('utf-8'))
        data = json_content['data'][0]
        return token, data


def update_record(token, module, record_id, data_object):
    url = f"https://www.zohoapis.com/crm/v2.1/{module}"
    headers = make_header(token)
    
    data_object["id"] = record_id

    request_body = dict()
    record_list = list()

    record_list.append(data_object)
    request_body['data'] = record_list

    data = json.dumps(request_body).encode('utf-8')
    response = requests.put(url=url, headers=headers, data=data)

    if response.status_code == 401:
        print("Auth issue")
        token.generate()
        return update_record(token, module, record_id, data_object)

    else:
        return token, response.status_code, json.loads(response.content.decode('utf-8'))


def convert_lead(token, record_id, **kwargs):
    data_object = kwargs
    request_body = dict()
    record_list = list()

    record_list.append(data_object)
    request_body['data'] = record_list

    data = json.dumps(request_body).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return convert_lead(token, record_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("data")

def create_record(token, module, data_object):
    url = f"https://www.zohoapis.com/crm/v2.1/{module}"
    headers = make_header(token)
    
    request_body = {}
    record_list = [data_object]
    request_body['data'] = record_list

    data = json.dumps(request_body).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return create_record(token, module, data_object)

    else:
        return token, response.status_code, json.loads(response.content.decode('utf-8'))


def get_records(token, module, **kwargs):
    url = f"https://www.zohoapis.com/crm/v2.1/{module}"
    headers = make_header(token)
    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return get_records(token, module, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        records = content.get('data')

        return token, records


def get_deleted_records(token, module, record_type, **kwargs):
    url = f'https://www.zohoapis.com/crm/v2.1/{module}/deleted?type={record_type}'
    headers = make_header(token)

    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        token.generate()
        return get_deleted_records(token, module, record_type, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('data')


def record_count(token, module, criteria=None, **kwargs):
    url = f'https://www.zohoapis.com/crm/v2.1/{module}/actions/count'
    headers = make_header(token)

    params = kwargs
    if criteria is not None:
        params['criteria': criteria]

    response = requests.get(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return record_count(token, module, criteria=criteria, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("count")

def search_records(token, module, criteria, **kwargs):
    url = f"https://www.zohoapis.com/crm/v2.1/{module}/search"
    headers = make_header(token)
    
    parameters = {"criteria":criteria}

    parameters.update(kwargs)

    response = requests.get(url=url, headers=headers, params=parameters)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return search_records(token, module, criteria, **kwargs)

    elif response.status_code >= 300:
        raise Exception("Error in response {}".format(response.status_code))

    else:
        content = json.loads(response.content.decode('utf-8'))
        records = content.get('data')
        page_info = content.get("info")
        return token, records, page_info


def mass_action(token, module, callback, **kwargs):
    empty = False
    url = f"https://www.zohoapis.com/crm/v2.1/{module}"
    headers = make_header(token)
    
    page = 1
    iterated = 0
    parameters = kwargs
    start_time = datetime.utcnow().isoformat()
    
    while not empty:
        parameters['page'] = str(page)
        parameters['per_page'] = "200"
        response = requests.get(url=url, headers=headers, params=parameters)
        if response.status_code == 401:
            token.generate()
            headers = make_header(token)
            continue
        content = json.loads(response.content.decode('utf-8'))
        data = content.get("data")

        if len(data) == 0:
            print("Done")
            empty = True

        for record in data:
            token, callback_response = callback(token, module, record)
            print(callback_response)
            iterated += 1
            print(f"{iterated} Records iterated")

        page += 1
        if len(data) < 200:
            empty = True
    end_time = datetime.utcnow().isoformat()
    return token, f"{iterated} Records iterated.\nStart Time: {start_time}\nEnd Time: {end_time}"


def list_action(token, module, callback, record_list):
    iterated = 0
    start_time = datetime.utcnow().isoformat()
    headers = make_header(token)

    for each record_id in record_list:
        url = f'https://www.zohoapis.com/crm/v2.1/{module}/{record_id}'
        token, callback_response = callback(token, module, record_id)

        iterated += 1
        print(callback_response)
        print(f"{iterated} Records Iterated")

    end_time = datetime.utcnow().isoformat()
    return token, f'{iterated} Records iterated.\nStart Time: {start_time}\nEnd Time: {end_time}'


        



import requests
import json


def make_header(token, org_id):
    return {
        "orgId": org_id,
        "Authorization": f"Zoho-oauthtoken {token.access}"
    }


def get_contact(token, org_id, contact_id, **kwargs):
    url = f'https://desk.zoho.com/api/v1/contacts/{contact_id}'
    headers = make_header(token, org_id)
    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return get_contact(token, org_id, contact_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content


def get_contacts(token, org_id, **kwargs):
    url = 'https://desk.zoho.com/api/v1/contacts'
    headers = make_header(token, org_id)
    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return get_contacts(token, org_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("data")


def create_contact(token, org_id, data_object):
    url = 'https://desk.zoho.com/api/v1/contacts'
    headers = make_header(token, org_id)
    data = json.dumps(data_object).encode('utf-8')
    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return create_contact(token, org_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content


def update_contact(token, org_id, contact_id, data_object):
    url = f'https://desk.zoho.com/api/v1/contacts/{contact_id}'
    headers = make_header(token, org_id)
    data = json.dumps(data_object).encode('utf-8')
    
    response = requests.patch(url=url, headers=headers, data=data)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return update_contact(token, org_id, contact_id, data_object)

    else:
        content =json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content


def contact_tickets(token, org_id, contact_id, **kwargs):
    url = f'https://desk.zoho.com/api/v1/contacts/{contact_id}/tickets'
    headers = make_header(token, org_id)
    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return contact_tickets(token, org_id, contact_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('data')


def contact_products(token, org_id, contact_id, **kwargs):
    url = f'https://desk.zoho.com/api/v1/contacts/{contact_id}/products'
    headers = make_header(token, org_id)
    response = requests.get(url=url, headers=headers, params=kwargs)
    if response.status_code == 401:
        print("Auth")
        token.generate()
        return contact_products(token, org_id, contact_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('data')


def contacts_count(token, org_id, view_id):
    url = 'https://desk.zoho.com/api/v1/contacts/count'
    headers = {
        'orgId':org_id,
        "Authorization": f'Zoho-oauthtoken {token.access}'
    }
    response = requests.get(url=url, headers=headers, params={'viewId':view_id})

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return contacts_count(token, org_id, view_id)
    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, int(content.get('count'))


def contact_stats(token, org_id, contact_id, **kwargs):
    url = f'https://desk.zoho.com/api/v1/contacts/{contact_id}/statistics'
    headers = make_header(token, org_id)
    response = requests.get(url=url, headers=headers, params=kwargs)
    
    if response.status_code == 401:
        print("Auth")
        token.generate()
        return contact_status(token, org_id, contact_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content


def contact_product_link(token, org_id, contact_id, product_id_list, associate=True, department_id=None):
    url = f'https://desk.zoho.com/api/v1/contacts/{contact_id}/associateProducts'
    headers = make_header(token, org_id)
    data_object = {"ids": product_id_list, 'associate': associate}

    data = json.dumps(data_object).encode('utf-8')
    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        print('Auth')
        token.generate()
        return contact_product_link(token, org_id, contact_id, product_id_list, associate=associate)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content.get('results')


def contact_accounts(token, org_id, contact_id, **kwargs):
    url = f'https://desk.zoho.com/api/v1/contacts/{contact_id}/accounts'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        token.generate()
        return contact_accounts(token, org_id, contact_id, **kwargs)

    else:
        content =json.loads(response.content.decode('utf-8'))

        return token, content.get('data')





    



import requests
import json


def format_header(token):
    return {
        "Authorization": f'Zoho-oauthtoken {token.access}'
    }


def create_contact(token, org_id, data_object):
    url = f'https://books.zoho.com/api/v3/contacts?organization_id={org_id}'
    headers = {
        'Authorization': f'Zoho-oauthtoken {token.access}'
    }
    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 400:
        token.generate()
        return create_contact(token, org_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content.get("contact")


def get_contacts(token, org_id, **kwargs):
    url = f'https://books.zoho.com/api/v3/contacts?organization_id={org_id}'
    headers = format_header(token)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 400:
        token.generate()
        return get_contacts(token, org_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        try:
            contacts = content['contacts']
            page_context = content['page_context']

        except Exception as e:
            contacts = [{'status': response.status_code, 'error': str(e)}]
            page_context = None

        return token, page_context, contacts



def update_contact(token, org_id, contact_id, data_object):
    url = f'https://books.zoho.com/api/v3/contacts/{contact_id}?organization_id={org_id}'
    headers = format_header(token)

    data = json.dumps(data_object).encode('utf-8')

    response = requests.put(url=url, headers=headers, data=data)

    if response.status_code == 400:
        token.generate()
        return update_contact(token, org_id, contact_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))

        try:
            contact = content['contact']
        except Exception as e:
            contact = {'status': response.status_code, 'error': str(e)}

        return token, response.status_code, contact


def get_contact(token, org_id, contact_id):
    url = f'https://books.zoho.com/api/v3/contacts/{contact_id}?organization_id={org_id}'
    headers = format_header(token)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 400:
        token.generate()
        return get_contact(token, org_id, contact_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        try:
            contact = content['contact']
        except Exception as e:
            contact = {'status': response.status_code, 'error': str(e)}

        return token, contact


def delete_contact(token, org_id, contact_id):
    url = f'https://books.zoho.com/api/v3/contacts/{contact_id}?organization_id={org_id}'
    headers = format_header(token)

    response = requests.delete(url=url, headers=headers)

    if response.status_code == 400:
        token.generate()
        return delete_contact(token, org_id, contact_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content['message']


def mark_status(token, org_id, contact_id, status):
    url = f'https://books.zoho.com/api/v3/contacts/{contact_id}/{status}?organization_id={org_id}'
    headers = format_header(token)

    response = requests.post(url=url, headers=headers)

    if response.status_code == 400:
        token.generate()
        return mark_status(token, org_id, contact_id, status)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content['message']


def grant_portal_access(token, org_id, contact_id, contact_person_ids):
    url = 'https://books.zoho.com/api/v3/contacts/{contact_id}/portal/enable?organization_id={org_id}'
    headers = format_header(token)

    data_object = {}
    contact_persons = []
    for id in contact_person_ids:
        obj = {"contact_person_id": id}
        contact_persons.append(obj)

    data_object['contact_persons'] = contact_persons

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 400:
        token.generate()
        return grant_portal_access(token, org_id, contact_id, contact_person_ids)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content['message']


def mass_action(token, org_id, callback, **kwargs):
    next_page = True
    page = 1
    iterated = 0
    parameters = kwargs
    while next_page:
        url = f'https://books.zoho.com/api/v3/contacts?organization_id={org_id}'
        headers = format_header(token)
        parameters['page'] = str(page)
        parameters['per_page'] = '200'

        response = requests.get(url=url, headers=headers, params=parameters)
        if response.status_code == 400:
            token.generate()
            continue

        content = json.loads(response.content.decode('utf-8'))
        contacts = content['contacts']

        for record in contacts:
            token, callback_response = callback(token, org_id, record)
            print(callback_response)
            iterated += 1
            print(f'{iterated} Records Iterated')

        page_context = content['page_context']
        page += 1
        next_page = page_context['has_more_page']
    return token, f'{iterated} Records Iterated \n=========\n Mass Operations Complete'



        




import requests
import json


def make_header(token):
    return {
        "Authorization": f"Zoho-oauthtoken {token.access}"
    }

def get_roles(token):
    url = 'https://www.zohoapis.com/crm/v2.1/settings/roles'
    headers = make_header(token)
    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_roles(token)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("roles")


def get_role(token, role_id):
    url = f'https://www.zohoapis.com/crm/v2.1/settings/roles/{role_id}'
    headers = make_header(token)
    
    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_role(token, role_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("roles")


def create_role(token, data_object):
    url = 'https://www.zohoapis.com/crm/v2.1/settings/roles'
    headers = make_header(token)
    request_body = dict()
    role_list = list()

    role_list.append(data_object)
    request_body['roles'] = role_list

    data = json.dumps(request_body).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return create_role(token, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content
    

def delete_role(token, role_id, transfer_to):
    url = f'https://www.zohoapis.com/crm/v2.1/settings/roles/{role_id}'
    params = {'transfer_to_id': transfer_to}
    headers = make_header(token)

    response = requests.delete(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return delete_role(token, role_id, transfer_to)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content


    



import requests
import json


def make_header(token, org_id):
    return {
        "orgId": org_id,
        "Authorization": f"Zoho-oauthtoken {token.access}"
    }


def get_ticket(token, org_id, ticket_id, **kwargs):
    url = 'https://desk.zoho.com/api/v1/tickets/{}'.format(ticket_id)
    headers = make_header(token, org_id)
    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return get_ticket(token, org_id, ticket_id, **kwargs)

    else:
        data = json.loads(response.content.decode('utf-8'))
        return token, data


def get_tickets(token, org_id, **kwargs):
    url = 'https://desk.zoho.com/api/v1/tickets'
    headers = make_header(token, org_id)
    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return get_tickets(token, org_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('data')


def create_ticket(token, org_id, data_object):
    url = 'https://desk.zoho.com/api/v1/tickets'
    headers = make_header(token, org_id)

    data = json.dumps(data_object).encode("utf-8")
    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return create_ticket(token, org_id, data_object)
    

def update_ticket(token, org_id, ticket_id, data_object):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}'
    data = json.dumps(data_object).encode('utf-8')
    headers = make_header(token, org_id)
    response = requests.patch(url=url, headers=headers, data=data)
    if response.status_code == 401:
        print("Auth")
        token.generate()
        return update_ticket(token, org_id, ticket_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content


def trash_ticket(token, org_id, ticket_id_list):
    url = 'https://desk.zoho.com/api/v1/tickets/moveToTrash'
    headers = make_header(token, org_id)
    
    data_object = {'ticketIds': ticket_id_list}

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return trash_ticket(token, org_id, ticket_id_list)

    else:
        return token, response.status_code


def move_ticket(token, org_id, ticket_id, department_id):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/move'
    headers = make_header(token, org_id)
    data_object = {'departmentId': department_id}

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return move_ticket(token, org_id, ticket_id, department_id)

    else:
        return token, response.status_code


def mark_read(token, org_id, ticket_id):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/markAsRead'
    headers = make_header(token, org_id)
    response = requests.post(url=url, headers=headers)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return mark_read(token, org_id, ticket_id)

    else:
        return token, response.status_code


def mark_unread(token, org_id, ticket_id):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/markAsUnRead'
    headers = make_header(token, org_id)
    response = requests.post(url=url, headers=headers)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return mark_unread(token, org_id, ticket_id)

    else:
        return token, reponse.status_code


def ticket_history(token, org_id, ticket_id, **kwargs):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/History'
    headers = make_header(token, org_id)
    
    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return ticket_history(token, org_id, ticket_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content.get('data')


def ticket_resolution(token, org_id, ticket_id):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/resolution'
    headers = make_header(token, org_id)
    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return ticket_resolution(token, org_id, ticket_id)

    



import requests
import json
import os
from datetime import datetime
import pickle


class Token:
    def __init__(self, client_id, client_secret, grant_token=None, refresh_token=None, save_refresh=True):
        self.client_id = client_id
        self.client_secret = client_secret
        self.access = None
        self.refresh = refresh_token
        if refresh_token is not None:
        	self.generate()
        if grant_token is not None:
            self.access, self.refresh = self._authorize(grant_token)
        if save_refresh:
        	self.write_to_file("token.pkl")
        
    def _authorize(self, grant_token):
        url = "https://accounts.zoho.com/oauth/v2/token"
        data = {
            "grant_type":"authorization_code",
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "code": grant_token
        }
        
        response = requests.post(url=url, data=data)
        print(response.status_code)
        if response.status_code == 200:
            content = json.loads(response.content.decode('utf-8'))
            print(content)
            access_token = content.get("access_token")
            refresh_token = content.get("refresh_token")
            self.auth_time = datetime.utcnow()
            self.valid_for = content.get("expires_in")
            return access_token, refresh_token
        else:
            raise Exception("Need to build exceptions")

    def _generate(self):
        url = "https://accounts.zoho.com/oauth/v2/token?refresh_token={}".format(self.refresh)
        url += "&client_id={}".format(self.client_id)
        url += "&client_secret={}".format(self.client_secret)
        url += "&grant_type=refresh_token"
	     
        response = requests.post(url=url)
        if response.status_code == 200:
            content = json.loads(response.content.decode('utf-8'))
            access_token = content.get("access_token")
            self.valid_for = content.get("expires_in")
            self.auth_time = datetime.utcnow()
            return access_token

    def generate(self):
        self.access = self._generate()      
       
    def write_to_env(self):
        os.environ["ZOHO_REFRESH_KEY"] = self.refresh

    def read_from_env(self):
        self.refresh = os.environ["ZOHO_REFRESH_KEY"]

	def write_to_file(self, filename):
		data_object = {
			"client_id": self.client_id,
			"client_secret": self.client_secret,
			"refresh_token": self.refresh
		}
		with open(filename, 'wb') as handle:
			pickle.dump(data_object, handle, protocol=pickle.HIGHEST_PROTOCOL)
			
	@classmethod
	def from_file(cls, filename):
		try:
			with open(filename, 'rb') as handle:
				data_object = pickle.load(handle)
			client_id = data_object['client_id']
			client_secret = data_object['client_secret']
			refresh = data_object['refresh']
		
		except FileNotFoundError:
			print("Invalid Filename"))
		
		except KeyError as e:
			print(str(e))
		
					
    
    @property
    def expires_in(self):
        now = datetime.utcnow()
        timedelta = now - self.auth_time

        seconds_in_use = int(timedelta.total_seconds())

        if seconds_in_use <= self.valid_for:
            return self.valid_for - seconds_in_use

        else:
            return None

    
            

        
        
        



import requests
import json


def make_header(token):
    return {
        'Authorization': f'Zoho-oauthtoken {token.access}'
    }


def get_notes(token, **kwargs):
    url = 'https://www.zohoapis.com/crm/v2.1/Notes'
    headers = make_header(token)
    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return get_notes(token, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("data")


def get_note(token, note_id):
    url = f'https://www.zohoapis.com/crm/v2.1/Notes/{note_id}'
    headers = make_header(token)
    
    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return get_note(token, note_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('data')


def get_record_notes(token, module, record_id, **kwargs):
    url = f'https://www.zohoapis.com/crm/v2.1/{module}/{record_id}/Notes'
    headers = make_header(token)

    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return get_record_notes(token, module, record_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('data')


def create_note(token, module, record_id, note_object):
    url = 'https://www.zohoapis.com/crm/v2.1/Notes'
    headers = make_header(token)

    request_body = {}
    note_object['Parent_Id'] = record_id
    note_object['se_module'] = module
    note_list = [note_object]

    request_body['data'] = record_list

    data = json.dumps(request_body).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if resonse.status_code == 401:
        print("Auth")
        token.generate()
        return create_note(token, module, record_id, note_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content.get('data')



def delete_note(token, module, note_id):
    url = f'https://www.zohoapis.com/crm/v2.1/Notes'
    headers = make_header(token)

    parameters = {'ids': note_id}

    response = requests.delete(url=url, headers=headers, params=parameters)

    if response.status_code == 401:
        token.generate()
        return delete_note(token, module, note_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('data')
